<?php
namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class SmAgrupamentoEscopoFactory
{

    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getAgrupamentoEscopoFaturamento()
    {

        $sql = "SELECT NRAGRUPAMENTOTIPOORCA, NMAGRUPAMENTOTIPOORCA, DSOBSAGRUPTIPOORCA, IDTPFATURA, CDFILIAL, IDNATUOPER, CDSERIE, CDTIPOOPERFATURA, CDINSCESTA, 
                            CDINSCMUNI, IDMUNIPRESTSERV, IDTIPONATUOPER, CDEMPRESA, CDEMPRESA || CDINSCESTA || CDINSCMUNI || CDSERIE AS NRSERIE
                    FROM FCT_AGRUPAMENTOTIPOORCA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function insertAgrupamentoTipoOrca($parametros)
    {
        $sql = "INSERT INTO FCT_AGRUPAMENTOTIPOORCA (NRAGRUPAMENTOTIPOORCA, NMAGRUPAMENTOTIPOORCA, DSOBSAGRUPTIPOORCA, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO, IDTPFATURA, CDFILIAL, CDTIPOOPERFATURA, IDNATUOPER, CDSERIE, CDINSCMUNI, CDINSCESTA, IDMUNIPRESTSERV, IDTIPONATUOPER, CDEMPRESA)
                VALUES (:NRAGRUPAMENTOTIPOORCA, :NMAGRUPAMENTOTIPOORCA, :DSOBSAGRUPTIPOORCA, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO, :IDTPFATURA, :CDFILIAL, :CDTIPOOPERFATURA, :IDNATUOPER, :CDSERIE, :CDINSCMUNI, :CDINSCESTA, :IDMUNIPRESTSERV, :IDTIPONATUOPER, :CDEMPRESA)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function updateAgrupamentoTipoOrca($parametros)
    {
        $sql = "UPDATE FCT_AGRUPAMENTOTIPOORCA
                    SET NMAGRUPAMENTOTIPOORCA   = :NMAGRUPAMENTOTIPOORCA,
                        DSOBSAGRUPTIPOORCA      = :DSOBSAGRUPTIPOORCA,
                        DTULTATU                = SYSDATE,
                        CDOPERULTATU            = :CDOPERULTATU,
                        NRORGULTATU             = :NRORGULTATU,
                        IDTPFATURA              = :IDTPFATURA,
                        CDFILIAL                = :CDFILIAL,
                        CDEMPRESA               = :CDEMPRESA,
                        CDTIPOOPERFATURA        = :CDTIPOOPERFATURA,                   
                        IDMUNIPRESTSERV         = :IDMUNIPRESTSERV,
                        IDNATUOPER              = :IDNATUOPER,
                        CDSERIE                 = :CDSERIE,
                        IDTIPONATUOPER          = :IDTIPONATUOPER,
                        CDINSCESTA              = :CDINSCESTA,
                        CDINSCMUNI              = :CDINSCMUNI
                    WHERE NRAGRUPAMENTOTIPOORCA = :NRAGRUPAMENTOTIPOORCA";
                    
        $this->connection->executeStatement($sql, $parametros);
    }

    public function getSerieNotaFiscal($parametros)
    {
        $sql = "SELECT CDEMPRESA, CDINSCESTA, CDINSCMUNI, CDSERIE, DSSERIE,  CDEMPRESA || CDINSCESTA || CDINSCMUNI || CDSERIE AS NRSERIE
                    FROM SERIENF
                    WHERE CDEMPRESA = :CDEMPRESA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getMunicipio($parametros)
    {
        $sql = "SELECT DISTINCT MUNI.CDPAIS, MUNI.SGESTADO, MUNI.CDMUNICIPIO, MUNICIPIO.NMMUNICIPIO, MUNICIPIO.NRORG
                    FROM (SELECT EM.CDPAIS, EM.SGESTADO, EM.CDMUNICIPIO
                            FROM EMPRESA EM
                            JOIN FILIAL FI ON EM.CDEMPRESA = FI.CDEMPRESA
                            WHERE FI.CDFILIAL = :CDFILIAL
                            AND FI.NRORG = :NRORG
                            AND FI.IDATIVO = 'S'
                            AND EM.IDATIVO = 'S'
        
                    UNION ALL
            
                    SELECT CDPAIS, SGESTADO, CDMUNICIPIO
                        FROM ENDERECOPARC
                        WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                        AND NRORG = :NRORG
                        AND CDTIPOENDERECO = 'PRINCIPAL'
                        AND IDATIVO = 'S'
                        AND CDMUNICIPIO IS NOT NULL
            
                    UNION ALL
                    
                    SELECT ENDE.CDPAIS, ENDE.SGESTADO, ENDE.CDMUNICIPIO
                    FROM ENDERECOPARC ENDE
                    JOIN ESTRUTURAM EST ON ENDE.NRPARCNEGOCIO = EST.NRPARCNEGOCIO
                    AND ENDE.NRORG = EST.NRORG
                    WHERE EST.NRESTRUTURAM = :NRESTRUTURAM
                    AND ENDE.NRORG = :NRORG
                    AND ENDE.CDTIPOENDERECO = 'PRINCIPAL'
                    AND ENDE.IDATIVO = 'S'
                    AND ENDE.CDMUNICIPIO IS NOT NULL) MUNI
                    JOIN MUNICIPIO ON MUNICIPIO.CDPAIS = MUNI.CDPAIS
                AND MUNICIPIO.SGESTADO = MUNI.SGESTADO
                AND MUNICIPIO.CDMUNICIPIO = MUNI.CDMUNICIPIO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getEscoposParaAgrupar($parametros)
    {
        $sql = "SELECT FCT_TIPOORCAMENTO.NRTIPOORCAMENTO, FCT_TIPOORCAMENTO.NMTIPOORCAMENTO  
                    FROM FCT_TIPOORCAMENTO
                    JOIN FCT_CAPAPARAMFATESC
                    ON FCT_CAPAPARAMFATESC.NRTIPOORCAMENTO = FCT_TIPOORCAMENTO.NRTIPOORCAMENTO
                    WHERE ((FCT_CAPAPARAMFATESC.IDTIPONATUOPER = :IDTIPONATUOPER) OR  ( (:IDTIPONATUOPER IS NULL) AND  (FCT_CAPAPARAMFATESC.IDTIPONATUOPER IS NULL)))
                    AND FCT_CAPAPARAMFATESC.CDFILIALFATURA = :CDFILIAL
                    AND FCT_CAPAPARAMFATESC.IDTPFATURA = :IDTPFATURA
                    AND FCT_CAPAPARAMFATESC.CDTIPOOPERFATURA = :CDTIPOOPER
                    AND FCT_CAPAPARAMFATESC.CDSERIENF = :CDSERIE
                    AND FCT_CAPAPARAMFATESC.CDINSCESTA = :CDINSCESTA
                    AND FCT_CAPAPARAMFATESC.CDINSCMUNI = :CDINSCMUNI
                    AND ((FCT_CAPAPARAMFATESC.IDMUNIPRESTSERV = :IDMUNIPRESTSERV) OR  ( (:IDMUNIPRESTSERV IS NULL) AND  (FCT_CAPAPARAMFATESC.IDMUNIPRESTSERV IS NULL)))
                    AND ((FCT_CAPAPARAMFATESC.IDNATUOPER = :IDNATUOPER) OR ((:IDNATUOPER IS NULL) AND  (FCT_CAPAPARAMFATESC.IDNATUOPER IS NULL)))
                    AND FCT_TIPOORCAMENTO.NRTIPOORCAMENTO NOT IN (SELECT NRTIPOORCAMENTO FROM FCT_RELACAGRUPTIPOORCA)";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getEscoposAgrupados($parametros)
    {
        $sql = "SELECT FCT_RELACAGRUPTIPOORCA.NRRELACAGRUPTIPOORCA, FCT_RELACAGRUPTIPOORCA.NRAGRUPAMENTOTIPOORCA, FCT_RELACAGRUPTIPOORCA.NRTIPOORCAMENTO, FCT_RELACAGRUPTIPOORCA.IDATIVO,
                FCT_TIPOORCAMENTO.NMTIPOORCAMENTO
                    FROM FCT_RELACAGRUPTIPOORCA
                    JOIN FCT_TIPOORCAMENTO
                    ON FCT_TIPOORCAMENTO.NRTIPOORCAMENTO = FCT_RELACAGRUPTIPOORCA.NRTIPOORCAMENTO
                    WHERE FCT_RELACAGRUPTIPOORCA.NRAGRUPAMENTOTIPOORCA = :NRAGRUPAMENTOTIPOORCA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function agruparEscopo($parametros)
    {
        $sql = "INSERT INTO FCT_RELACAGRUPTIPOORCA (NRRELACAGRUPTIPOORCA, NRAGRUPAMENTOTIPOORCA, NRTIPOORCAMENTO, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
                VALUES (:NRRELACAGRUPTIPOORCA, :NRAGRUPAMENTOTIPOORCA, :NRTIPOORCAMENTO, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";
    
        $this->connection->executeStatement($sql, $parametros);
    }

    public function excluirEscoposAgrupados($parametros)
    {
        $sql = "DELETE FROM FCT_RELACAGRUPTIPOORCA
                    WHERE NRRELACAGRUPTIPOORCA IN ($parametros)";
            
        $this->connection->executeStatement($sql);
    }
}
